﻿using System.Runtime.Serialization.Json;
using System.Text;
using System.Xml;

namespace CoreLibrary.Persistance
{
    /// <summary>
    /// Classe PersistanceJSON, implémentant l'interface IPersistance.
    /// </summary>
    public class PersistanceJSON : IPersistance
    {
        /// <summary>
        /// Chaîne de caractères représentant le nom du dossier pour sauvegarder et charger.
        /// </summary>
        private readonly string nomDossier = "Fichiers";

        /// <summary>
        /// Chaîne de caractères représentant le nom du doissier à la racine.
        /// </summary>
        private readonly string nomRacine = "Sources";

        /// <summary>
        /// Chaîne de caractères représentant le nom du dossier courant.
        /// </summary>
        private readonly string dossier = AppDomain.CurrentDomain.BaseDirectory;

        /// <summary>
        /// Constructeur de PersistanceJSON.
        /// </summary>
        public PersistanceJSON()
        {
            while (new DirectoryInfo(dossier).Name != nomRacine)
                dossier = Path.GetFullPath(Path.Combine(dossier, @"../"));
        }

        /// <summary>
        /// Charge les données trouvés dans le fichier de sauvegarde.
        /// </summary>
        /// <returns>Renvoie un tableau de tous les éléments chargés.</returns>
        public T[] Charger<T>() where T : IEstPersistant
        {
            string fichier = $"{typeof(T).Name.ToLower()}s.json";

            Directory.SetCurrentDirectory(dossier);

            if (!Directory.Exists(nomDossier))
                return [];

            Directory.SetCurrentDirectory(Path.Combine(Directory.GetCurrentDirectory(), nomDossier));
            var directory = Directory.GetCurrentDirectory();

            if (!File.Exists(fichier))
                return [];

            DataContractJsonSerializer jsonSerializer = new DataContractJsonSerializer(typeof(T[]));
            T[] elements;

            using (FileStream s = File.OpenRead(fichier))
            {
                elements = jsonSerializer.ReadObject(s) as T[];
            }

            return elements;
        }

        /// <summary>
        /// Enragistre dans le fichier de sauvegarde, les éléments spécifiés.
        /// </summary>
        /// <param name="elements">Elements à enregistrer dans le fichier de sauvegarde.</param>
        public void Enregistrer<T>(T[] elements) where T : IEstPersistant
        {
            string fichier = $"{typeof(T).Name.ToLower()}s.json";

            Directory.SetCurrentDirectory(dossier);

            if (!Directory.Exists(nomDossier))
                Directory.CreateDirectory(nomDossier);

            Directory.SetCurrentDirectory(Path.Combine(Directory.GetCurrentDirectory(), nomDossier));

            DataContractJsonSerializer jsonSerializer = new DataContractJsonSerializer(typeof(T[]));

            using (FileStream s = File.Create(fichier))
            {
                using (XmlDictionaryWriter writer = JsonReaderWriterFactory.CreateJsonWriter(
                                        s,
                                        Encoding.UTF8,
                                        false,
                                        true))
                {
                    jsonSerializer.WriteObject(writer, elements);
                }
            }
        }
    }
}
